import operator

from cloud_governance.common.aws.cloudtrail.cloudtrail_operations import CloudTrailOperations
from cloud_governance.common.logger.init_logger import logger
from cloud_governance.zombie_non_cluster.run_zombie_non_cluster_policies import NonClusterZombiePolicy
from operator import ge


class EC2Stop(NonClusterZombiePolicy):
    """
    This class delete the stopped ec2 instance
    Send mail to the user if the ec2 instance is stopped > 20, 25th days
    Delete the instance when it stopped 30 days
    """

    FIRST_MAIL_NOTIFICATION_INSTANCE_DAYS = 20
    SECOND_MAIL_NOTIFICATION_INSTANCE_DAYS = 25
    DELETE_INSTANCE_DAYS = 30

    def __init__(self):
        super().__init__()
        self._cloudtrail = CloudTrailOperations(region_name=self._region)

    def run(self):
        """
        This method list all stopped instances for more than 30 days and terminate if dry_run no
        @return:
        """
        return self.__fetch_stop_instance(sign=ge, instance_days=self.FIRST_MAIL_NOTIFICATION_INSTANCE_DAYS, delete_instance_days=self.DELETE_INSTANCE_DAYS)

    def __fetch_stop_instance(self, instance_days: int, delete_instance_days: int, sign: operator = ge):
        """
        This method list all stopped instances for more than 30 days and terminate if dry_run no
        @return:
        """
        instances = self._ec2_client.describe_instances(Filters=[{'Name': 'instance-state-name', 'Values': ['stopped']}])['Reservations']
        stopped_instances = []
        stopped_instance_tags = {}
        for instance in instances:
            for resource in instance['Instances']:
                instance_id = resource.get('InstanceId')
                stopped_time = self._cloudtrail.get_stop_time(resource_id=instance_id, event_name='StopInstances')
                if not stopped_time:
                    stopped_time = resource.get('LaunchTime')
                days = self._calculate_days(create_date=stopped_time)
                if days in (instance_days, self.SECOND_MAIL_NOTIFICATION_INSTANCE_DAYS):
                    user = self._get_tag_name_from_tags(tags=resource.get('Tags'), tag_name='User')
                    if user:
                        self.__trigger_mail(user=user, stopped_time=stopped_time, resource_id=instance_id, days=days)
                    else:
                        logger.info('User is missing')
                if sign(days, instance_days):
                    if days == delete_instance_days:
                        stopped_instance_tags[resource.get('InstanceId')] = resource.get('Tags')
                    stopped_instances.append([resource.get('InstanceId'), self._get_tag_name_from_tags(tags=resource.get('Tags'), tag_name='Name'),
                                              self._get_tag_name_from_tags(tags=resource.get('Tags'), tag_name='User'), str(resource.get('LaunchTime')),
                                              self._get_tag_name_from_tags(tags=resource.get('Tags'), tag_name='Policy')
                                              ])
        if self._dry_run == "no":
            for instance_id, tags in stopped_instance_tags.items():
                if self._get_policy_value(tags=tags) != 'NOTDELETE':
                    tag_specifications = [{'ResourceType': 'image', 'Tags': tags}]
                    if sign == ge:
                        tag_specifications.append({'ResourceType': 'snapshot', 'Tags': tags})
                    self._ec2_client.create_image(InstanceId=instance_id, Name=self._get_tag_name_from_tags(tags=tags), TagSpecifications=tag_specifications)
                    self._ec2_client.terminate_instances(InstanceIds=[instance_id])
                    logger.info(f'Deleted the instance: {instance_id}')
        return stopped_instances

    def __trigger_mail(self, user: str, stopped_time: str, resource_id: str, days: int):
        """
        This method send triggering mail
        @param user:
        @param stopped_time:
        @param resource_id:
        @param days:
        @return:
        """
        if user in self._literal_eval():
            receivers_list = [f'{self._literal_eval()[user]}@redhat.com']
        else:
            receivers_list = [f'{user}@redhat.com']
        subject = f'Instance- {resource_id} stopped more than {days} days'
        body = f"""
Hi,

Instance: {resource_id} is stopped on {stopped_time} is stopped more than {days} days.  
After {self.DELETE_INSTANCE_DAYS} days of instance is stopped we delete that instance. Please backup your data. If you don't want to delete this instance add Policy=Not_Delete tag to your instance.
If you already added the Policy=Not_Delete tag ignore this mail.

Thanks
Thirumalesh""".strip()
        self._mail.send_mail(receivers_list=receivers_list, body=body, subject=subject)



import boto3

from cloud_governance.common.aws.cloudtrail.cloudtrail_operations import CloudTrailOperations
from cloud_governance.common.aws.ec2.ec2_operations import EC2Operations
from cloud_governance.common.aws.iam.iam_operations import IAMOperations
from cloud_governance.common.aws.utils.utils import Utils


class TagClusterOperations:

    def __init__(self, input_tags: dict,  cluster_name: str,  cluster_prefix: str, region: str, dry_run: str, cluster_only: bool):
        self.cluster_only = cluster_only
        self.cluster_prefix = cluster_prefix
        self.utils = Utils(region=region)
        self.ec2_client = boto3.client('ec2', region_name=region)
        self.elb_client = boto3.client('elb', region_name=region)
        self.elbv2_client = boto3.client('elbv2', region_name=region)
        self.iam_client = boto3.client('iam', region_name=region)
        self.s3_client = boto3.client('s3')
        self.ec2_operations = EC2Operations(region=region)
        self.iam_operations = IAMOperations()
        self.cluster_name = cluster_name
        self.input_tags = input_tags
        self.cloudtrail = CloudTrailOperations(region_name='us-east-1')
        self._get_username_from_instance_id_and_time = CloudTrailOperations(region_name=region).get_username_by_instance_id_and_time
        self.dry_run = dry_run

    def _input_tags_list_builder(self):
        """
        This method build tags list according to input tags dictionary
        @return:
        """
        tags_list = []
        for key, value in self.input_tags.items():
            tags_list.append({'Key': key, 'Value': value})
        return tags_list

    def _get_instances_data(self):
        """
        This method go over all instances
        @return:
        """
        instances_list = []
        ec2s_data = self.ec2_operations.get_instances()
        for items in ec2s_data:
            if items.get('Instances'):
                instances_list.append(items['Instances'])
        return instances_list

    def _fill_na_tags(self, user: str = None):
        """
        This method fill the NA tags
        @param user:
        @return:
        """
        keys = ['User', 'Owner', 'Manager', 'Project', 'Environment', 'Email']
        tags = []
        value = 'NA'
        for key in keys:
            if user and user == 'User':
                value = user
            elif user and user == 'Email':
                value = f'{user}@redhat.com'
            else:
                tags.append({'Key': key, 'Value': value})
        return tags



import json
from datetime import datetime, timedelta

import boto3


class CostExplorerOperations:
    """
    This class extracts the price data from AWS Cost Explorer
    """

    def __init__(self):
        self.cost_explorer_client = boto3.client('ce')

    def get_cost_by_tags(self, tag: str, granularity: str = 'DAILY', cost_metric: str = 'UnblendedCost', start_date: str = '', end_date: str = ''):
        """
        This method extracts the price by Tag provided
        @return:
        """
        if not start_date and not end_date:
            end_date = datetime.now()
            start_date = end_date - timedelta(1)
            start_date = str(start_date.strftime('%Y-%m-%d'))
            end_date = str(end_date.strftime('%Y-%m-%d'))
        return self.cost_explorer_client.get_cost_and_usage(
            TimePeriod={
                'Start': start_date,
                'End': end_date
            },
            Granularity=granularity,
            Metrics=[cost_metric],
            GroupBy=[{'Type': 'TAG', 'Key': tag}]
        )



import csv

import boto3

from cloud_governance.common.aws.ec2.ec2_operations import EC2Operations
from cloud_governance.common.logger.init_logger import logger


class UpdateNATags:

    def __init__(self, file_name: str, region: str = 'us-east-2'):
        self.region = region
        self.ec2_client = boto3.client('ec2', region_name=region)
        self.file_name = file_name
        self.ec2_operations = EC2Operations(region=region)

    def __get_resource_data(self, resource: str):
        """
        This method gives resource data
        @return:
        """
        if resource == 'instance':
            instances_data = self.ec2_operations.get_instances()
            instances = []
            for instance in instances_data:
                for resource in instance['Instances']:
                    instances.append(resource)
            return instances
        elif resource == 'volume':
            return self.ec2_operations.get_volumes()
        elif resource == 'snapshot':
            return self.ec2_operations.get_snapshots()
        elif resource == 'image':
            return self.ec2_operations.get_images()

    def __get_resource_ids(self, resource_id: str, resource_name: str):
        """
        This method get all instances
        @return:
        """
        resources = self.__get_resource_data(resource=resource_name)
        resource_ids = []
        for resource in resources:
            resource_ids.append(resource.get(resource_id))
        return resource_ids

    def __get_key_value(self, key, value):
        """
        This method return key-value pairs
        :param key:
        :param value:
        :return:
        """
        return {'Key': key, 'Value': value}

    def __convert_key_value(self, keys, values):
        """
        This return dict of resource and its tags
        :param keys:
        :param values:
        :return: 
        """
        tags = {}
        for value in values:
            tags[value[0]] = []
            for index, tag_value in enumerate(value[1:]):
                if tag_value.strip():
                    tags[value[0]].append(self.__get_key_value(keys[index].strip(), tag_value.strip()))
        return tags

    def __beautify_csv_data_to_key_value_pairs(self):
        """
        This method convert the csv data to key-value pairs
        :return:
        """
        with open(self.file_name) as file:
            csvreader = csv.reader(file)
            header = next(csvreader)[1:]
            rows = []
            for row in csvreader:
                rows.append(row)
            return self.__convert_key_value(header, rows)

    def update_tags(self):
        """
        This method update tags
        @return:
        """
        instances = self.__get_resource_ids(resource_id='InstanceId', resource_name='instance')
        volumes = self.__get_resource_ids(resource_id='VolumeId', resource_name='volume')
        snapshots = self.__get_resource_ids(resource_id='SnapshotId', resource_name='snapshot')
        images = self.__get_resource_ids(resource_id='ImageId', resource_name='image')
        na_resources = self.__beautify_csv_data_to_key_value_pairs()
        co = 0
        for resource, resource_tags in na_resources.items():
            if resource in instances or resource in volumes or resource in snapshots or resource in images:
                co += 1
                self.ec2_client.create_tags(Resources=[resource], Tags=resource_tags)
                logger.info(f'Updated the Tags of {resource}')

        return co

    def __extract_tags(self, tags):
        """
        This method extract tags from the resources
        @param tags:
        @return:
        """
        extract_tags = {}
        tags_list = []
        for tag in tags:
            if not tag.get('Key').startswith('kubernetes.io') and not tag.get('Key').startswith('aws:'):
                extract_tags[tag.get('Key')] = tag.get('Value')
                tags_list.append(tag.get('Key'))
        return [extract_tags, tags_list]

    def __resource_list(self, resource_id: str, resource_name: str):
        """
        This method returns resource_id and tags
        @param resource_id:
        @param resource_name:
        @return:
        """
        resource_ids = {}
        resource_data = self.__get_resource_data(resource=resource_name)
        keys = []
        for resource in resource_data:
            if resource.get('Tags'):
                for tag in resource.get('Tags'):
                    if tag.get("Key") == "User" and tag.get('Value') == 'NA':
                        tags, tag_keys = self.__extract_tags(resource.get('Tags'))
                        resource_ids[resource.get(resource_id)] = tags
                        keys.extend(tag_keys)
                        break
        return [resource_ids, set(keys)]

    def create_csv(self):
        with open(f'{self.file_name}', 'w') as file:
            instances, instance_tags = self.__resource_list(resource_id='InstanceId', resource_name='instance')
            volumes, volume_tags = self.__resource_list(resource_id='VolumeId', resource_name='volume')
            snapshots, snapshot_tags = self.__resource_list(resource_id='SnapshotId', resource_name='snapshot')
            images, images_tags = self.__resource_list(resource_id='ImageId', resource_name='image')
            resources = [instances, volumes, snapshots, images]
            tags = list(set(list(instance_tags) + list(volume_tags) + list(snapshot_tags) + list(images_tags)))
            tags.sort()
            file.write('Resource_Id, ')
            for tag in tags:
                file.write(f'{tag}, ')
            file.write('\n')
            for resource in resources:
                for instance, key_tags in resource.items():
                    file.write(f'{instance}, ')
                    for value in sorted(key_tags):
                        if key_tags.get(value).strip():
                            file.write(f'{key_tags.get(value).strip()}, ')
                        else:
                            file.write(f', ')
                    file.write('\n')
